/******************************************************************
 ***************   Copyright  (C)  WCH  2004-2023   ***************
 ***************         Web: http://wch.cn         ***************
 ***************      Author: WCH <tech@wch.cn>     ***************
 ***************                                    ***************
 ***************         IoCHub library Demo        ***************
 ******************************************************************
 * *************           南京沁恒微电子            ***************
 * ****************************************************************/

/********************************************
 * IoCHubLibrary Demo                       *
 * 此Demo接口能使用异步方式的都会使用异步方式    *
 * 此Demo仅支持最多打开一个会话                *
 ********************************************/

package cn.wch.iochublibrarydemo;

import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Toast;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;
import cn.wch.iochublibrary.WCHIoCHubManager;
import cn.wch.iochublibrary.config.InitConfig;
import cn.wch.iochublibrary.error.IoCHubException;

public class InitConfigDialog extends Dialog{

    @BindView(R.id.dataLzo)
    CheckBox dataLzo;
    @BindView(R.id.deviceBond)
    CheckBox deviceBond;
    @BindView(R.id.receiveBufferSize)
    EditText receiveBufferSize;
    @BindView(R.id.sendBufferSize)
    EditText sendBufferSize;
    @BindView(R.id.maxConnSize)
    EditText maxConnSize;
    @BindView(R.id.sendSizePerPackage)
    EditText sendSizePerPackage;
    @BindView(R.id.socketWndSize)
    EditText socketWndSize;
    @BindView(R.id.socketWndTimeout)
    EditText socketWndTimeout;
    @BindView(R.id.cancelBtn)
    Button cancelBtn;
    @BindView(R.id.confirmBtn)
    Button confirmBtn;

    private Unbinder unbinder;
    private Context context;

    public static InitConfigDialog newInstance(Context context){
        InitConfigDialog initConfigDialog = new InitConfigDialog(context);
        return initConfigDialog;
    }

    InitConfigDialog(Context context){
        super(context);
        this.context = context;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.init_conig);
        unbinder = ButterKnife.bind(this);

        showView();
        setClickedListener();
    }

    private void showView(){
        try {
            InitConfig initConfig = WCHIoCHubManager.getInstance().getConfig();
            dataLzo.setChecked(initConfig.isSupportCompress());
            deviceBond.setChecked(initConfig.isOpenDeviceLock());
            receiveBufferSize.setText(String.format("%d",initConfig.getReceiveBufferSize()));
            sendBufferSize.setText(String.format("%d",initConfig.getSendBufferSize()));
            maxConnSize.setText(String.format("%d",initConfig.getMaxConnSize()));
            sendSizePerPackage.setText(String.format("%d",initConfig.getSendSizePerPackage()));
            socketWndSize.setText(String.format("%d",initConfig.getSocketWndSize()));
            socketWndTimeout.setText(String.format("%d",initConfig.getSocketWndTimeout()));
        } catch (IoCHubException e) {
            showToast("获取配置失败:" + e.getErrorCode());
        }
    }

    private void setClickedListener(){

        cancelBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss();
            }
        });

        confirmBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                confirm();
            }
        });
    }

    private void confirm(){
        if (!checkEmpty()){
            return;
        }
        InitConfig initConfig = new InitConfig.Builder()
                .receiveBufferSize(Integer.parseInt(receiveBufferSize.getText().toString()))
                .sendBufferSize(Integer.parseInt(sendBufferSize.getText().toString()))
                .maxConnSize(Integer.parseInt(maxConnSize.getText().toString()))
                .sendSizePerPackage(Integer.parseInt(sendSizePerPackage.getText().toString()))
                .socketWndSize(Integer.parseInt(socketWndSize.getText().toString()))
                .socketWndTimeout(Integer.parseInt(socketWndTimeout.getText().toString()))
                .supportCompress(dataLzo.isChecked())
                .openDeviceLock(deviceBond.isChecked())
                .build();
        int ret = WCHIoCHubManager.getInstance().setConfig(initConfig);
        if (ret != 0){
            showToast("参数配置失败:" + ret);
        }
        dismiss();
    }

    private boolean checkEmpty(){
        if (receiveBufferSize.getText().length() == 0){
            showToast("未输入接收缓冲区大小");
            return false;
        }
        if (sendBufferSize.getText().length() == 0){
            showToast("未输入发送缓冲区大小");
            return false;
        }
        if (maxConnSize.getText().length() == 0){
            showToast("未输入最大连接数");
            return false;
        }
        if (sendSizePerPackage.getText().length() == 0){
            showToast("未输入每包发送大小");
            return false;
        }
        if (socketWndSize.getText().length() == 0){
            showToast("未输入窗口大小");
            return false;
        }
        if (socketWndTimeout.getText().length() == 0){
            showToast("未输入窗口应答超时时间");
            return false;
        }
        return true;
    }

    @Override
    public void dismiss() {
        super.dismiss();
        unbinder.unbind();
    }

    private void showToast(String message){
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(context,message,Toast.LENGTH_SHORT).show();
            }
        });
    }
}
