/******************************************************************
 ***************   Copyright  (C)  WCH  2004-2023   ***************
 ***************         Web: http://wch.cn         ***************
 ***************      Author: WCH <tech@wch.cn>     ***************
 ***************                                    ***************
 ***************         IoCHub library Demo        ***************
 ******************************************************************
 * *************           南京沁恒微电子            ***************
 * ****************************************************************/

/********************************************
 * IoCHubLibrary Demo                       *
 * 此Demo接口能使用异步方式的都会使用异步方式    *
 * 此Demo仅支持最多打开一个会话                *
 ********************************************/
package cn.wch.iochublibrarydemo;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.method.ScrollingMovementMethod;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.touchmcu.ui.DialogUtil;

import java.net.Socket;
import java.nio.channels.SocketChannel;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;
import cn.wch.iochublibrary.WCHIoCHubManager;
import cn.wch.iochublibrary.callback.EventListener;
import cn.wch.iochublibrary.callback.LANScanCallback;
import cn.wch.iochublibrary.config.InitConfig;
import cn.wch.iochublibrary.error.IoCHubException;
import cn.wch.iochublibrary.lan.LANDevice;


public class MainActivity extends AppCompatActivity {

    @BindView(R.id.activityToolbar)
    Toolbar toolbar;
    @BindView(R.id.lanDeviceSpinner)
    Spinner lanDeviceSpinner;
    @BindView(R.id.lanScanBtn)
    Button lanScanBtn;
    @BindView(R.id.targetNodeID)
    TextView targetNodeID;
    @BindView(R.id.targetSessionKey)
    TextView targetSessionKey;
    @BindView(R.id.openSessionBtn)
    Button openSessionBtn;
    @BindView(R.id.recvSizeText)
    TextView recvSizeText;
    @BindView(R.id.recvSpeedText)
    TextView recvSpeedText;
    @BindView(R.id.recvShowCheckBox)
    CheckBox recvShowCheckBox;
    @BindView(R.id.recvDataText)
    TextView recvDataText;
    @BindView(R.id.recvClearBtn)
    Button recvClearBtn;
    @BindView(R.id.sendSizeText)
    TextView sendSizeText;
    @BindView(R.id.sendSpeedText)
    TextView sendSpeedText;
    @BindView(R.id.autoSendCheckBox)
    CheckBox autoSendCheckBox;
    @BindView(R.id.interval)
    EditText intervalEdit;
    @BindView(R.id.sendDataEdit)
    EditText sendDataEdit;
    @BindView(R.id.sendClearBtn)
    Button sendClearBtn;
    @BindView(R.id.sendBtn)
    Button sendBtn;

    private Unbinder unbinder;
    private Menu menu;

    private boolean isInit = false;   //是否初始化库
    private boolean isStart = false;  //是否启动

    private List<String> lanDeviceList;
    private ArrayAdapter<String> lanDeviceAdapter;

    private int openSessionFd;   //会话标识
    private String openNodeID;   //NodeID

    private long receiveSize,sendSize,copyRecvSize,copySendSize;

    private Timer speedTimer;
    private TimerTask speedTimerTask;
    private boolean isSpeed = false;

    private Timer autoTimer;
    private TimerTask autoTimerTask;
    private boolean isAuto = false;

    private final int START_SUCCESS = 0;  //启动成功
    private final int LAN_SCAN_SUCCESS = 1;  //局域网扫描成功
    private final int OPEN_SESSION_SUCCESS = 2;  //打开会话成功
    private final int SESSION_CLOSED = 3;  //会话被关闭
    private final int SESSION_READ = 4;  //读取到数据

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        unbinder = ButterKnife.bind(this);

        intVariable();
        initView();
        initIoCHubLib();
        getLocalNodeID();
        setBtnClickListener();
    }

    //初始化参数
    private void intVariable(){
        lanDeviceList = new ArrayList<>();
        lanDeviceAdapter = new ArrayAdapter<String>(MainActivity.this,R.layout.spinner_style, lanDeviceList);
    }

    //初始化界面
    private void initView(){
        setSupportActionBar(toolbar);
        getSupportActionBar().setTitle("IoChubLibraryDemo");
        recvDataText.setMovementMethod(ScrollingMovementMethod.getInstance());  //TextView滚动显示
        setIOViewEnable(false);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu, menu);
        this.menu = menu;
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        int itemId = item.getItemId();
        switch (itemId){
            case R.id.start:
                start();
                break;
            case R.id.stop:
                stop();
                break;
            case R.id.config:
                config();
                break;
            case R.id.accessMode:
                accessMode();
                break;
            case R.id.transferType:
                getTransferType();
                break;
            case R.id.natType:
                getNATType();
                break;
            case R.id.version:
                getVersion();
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    private void setMenu(){
        menu.findItem(R.id.start).setVisible(!isStart);
        menu.findItem(R.id.stop).setVisible(isStart);
    }

    //初始化库
    private void initIoCHubLib(){
        int ret = WCHIoCHubManager.getInstance().init(MainActivity.this);
        if (ret != 0){
            showToast("初始化库失败:" + ret);
            return;
        }
        isInit = true;
        WCHIoCHubManager.getInstance().registerEventListener(eventListener);
        InitConfig initConfig = new InitConfig.Builder()
                .maxConnSize(1)
                .build();
        WCHIoCHubManager.getInstance().setConfig(initConfig);
    }

    //获取本机NodeID并显示
    private void getLocalNodeID(){
        String nodeID = WCHIoCHubManager.getInstance().getLocalID();
        if (nodeID == null){
            return;
        }
        getSupportActionBar().setTitle(nodeID);
    }

    //事件回调，此Demo接口皆使用异步方式
    private EventListener eventListener = new EventListener() {
        @Override
        public void onStartResult(boolean result, int code) {
            if (result){
                sendMessage(START_SUCCESS);
            } else {
                showToast("启动失败:" + code);
            }
        }

        @Override
        public void onSessionOpenSuccess(int sessionFd, String nodeID) {
            openSessionFd = sessionFd;
            openNodeID = nodeID;
            showToast("会话打开成功");
            sendMessage(OPEN_SESSION_SUCCESS);
        }

        @Override
        public void onSessionOpenFailure(String nodeID, int code) {
            showToast("会话打开失败" + code);
        }

        @Override
        public void onSessionRead(int sessionFd, int readLen) {
            sessionRead(sessionFd,readLen);
        }

        @Override
        public void onSessionTransferType(int sessionFd, int type) {
            showToast("会话通信方式切换为:" + type);
        }

        @Override
        public void onSessionException(int sessionFd) {
            showToast("会话异常");
        }

        @Override
        public void onSessionClose(int sessionFd, String nodeID) {
            showToast("会话被关闭:" + nodeID);
            sendMessage(SESSION_CLOSED);
        }

        @Override
        public void onException(int code) {
            showToast("发生异常:" + code);
        }
    };

    //启动
    private void start(){
        if (!isInit){
            showToast("初始化库失败");
            return;
        }
        try {
            WCHIoCHubManager.getInstance().start("58.213.74.190:38089");  //异步
        } catch (IoCHubException e) {
            showToast("启动失败:" + e.getErrorCode());
        }
    }

    //停止
    private void stop(){
        WCHIoCHubManager.getInstance().stop();
        isStart = false;
        setMenu();
    }

    //配置参数
    private void config(){
        if (!isInit){
            showToast("初始化库失败");
            return;
        }
        if (isStart){
            showToast("启动前配置参数");
            return;
        }
        InitConfigDialog initConfigDialog = InitConfigDialog.newInstance(MainActivity.this);
        initConfigDialog.setCancelable(false);
        initConfigDialog.show();
    }

    //设置会话模式
    private void accessMode(){
        if (!isInit){
            showToast("初始化库失败");

            return;
        }
        AccessModeDialog accessModeDialog = AccessModeDialog.newInstance(MainActivity.this);
        accessModeDialog.setCancelable(false);
        accessModeDialog.show();
    }

    //获取会话的通信方式
    private void getTransferType(){
        if (!isInit){
            showToast("初始化库失败");
            return;
        }
        int ret = WCHIoCHubManager.getInstance().getTransferType(openSessionFd);
        if (ret == 0x00){
            showToast("未打开会话");
        } else if (ret == 0x01){
            showToast("通信类型:P2P");
        } else if (ret == 0x02){
            showToast("通信类型:转发");
        } else if (ret == 0x03){
            showToast("通信类型:局域网");
        }
    }

    //获取当前网络的NAT类型
    private void getNATType(){
        if (!isInit){
            showToast("初始化库失败");
            return;
        }
        int ret = WCHIoCHubManager.getInstance().getNATType();
        if (ret == 0x00){
            showToast("NAT类型:非对称型");
        } else if (ret == 0x10){
            showToast("NAT类型:对称型");
        } else if (ret == 0xFF){
            showToast("未启动IoCHub");
        }
    }

    //获取库版本
    private void getVersion(){
        String version = WCHIoCHubManager.getInstance().getSDKVersion();
        showToast("当前版本:" + version);
    }

    private void setBtnClickListener(){

        lanScanBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                lanScan();
            }
        });

        lanDeviceSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                String nodeID = parent.getItemAtPosition(position).toString();
                targetNodeID.setText(nodeID);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        openSessionBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (openSessionBtn.getText().toString().equalsIgnoreCase("打开会话")){
                    openSession();
                } else if (openSessionBtn.getText().toString().equalsIgnoreCase("关闭会话")){
                    closeSession();
                }
            }
        });

        recvClearBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                receiveSize = 0;
                recvSizeText.setText("接收:0B");
                recvSpeedText.setText("速度:0B/s");
                recvDataText.setText("");
                recvDataText.scrollTo(0,0);
            }
        });

        sendClearBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                sendSize = 0;
                sendSizeText.setText("发送:0B");
                sendSpeedText.setText("速度:0B/s");
                sendDataEdit.setText("");
            }
        });

        sendBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (sendBtn.getText().toString().equalsIgnoreCase("发送")){
                    sessionWrite();
                } else if (sendBtn.getText().toString().equalsIgnoreCase("停止")){
                    stopAutoSendTimer();
                    sendBtn.setText("发送");
                }
            }
        });
    }

    //局域网扫描
    private void lanScan(){
        if (!isInit){
            showToast("初始化库失败");
            return;
        }
        int ret = WCHIoCHubManager.getInstance().scanLanNodes(3000, new LANScanCallback() {
            @Override
            public void onScanResult(ArrayList<LANDevice> deviceList) {
                DialogUtil.getInstance().hideLoadingDialog();
                if (deviceList.size() == 0){
                    return;
                }
                lanDeviceList.clear();
                for (LANDevice lanDevice:deviceList){
                    lanDeviceList.add(lanDevice.getNodeID());
                }
                sendMessage(LAN_SCAN_SUCCESS);
            }
        });
        if (ret == 0){
            DialogUtil.getInstance().showLoadingDialog(MainActivity.this,"正在扫描");
        } else {
            showToast("扫描失败:" + ret);
        }
    }

    //打开会话
    private void openSession(){
        if (!isInit){
            showToast("初始化库失败");
            return;
        }
        if (targetNodeID.getText().length() == 0){
            showToast("未输入待连接NodeID");
            return;
        }
        if (targetNodeID.getText().length() != 16){
            showToast("待连接NodeID不规范");
            return;
        }
        String nodeID = targetNodeID.getText().toString();
        if (targetSessionKey.getText().length() > 8){
            showToast("会话秘钥不规范");
            return;
        }
        String sessionKey = null;
        if (targetSessionKey.getText().length() != 0){
            sessionKey = targetSessionKey.getText().toString();
        }
        try {
            WCHIoCHubManager.getInstance().sessionOpen(nodeID,sessionKey);
        } catch (IoCHubException e) {
            showToast("打开会话失败:" + e.getErrorCode());
        }
    }

    //关闭会话
    private void closeSession(){
        stopAutoSendTimer();
        sendBtn.setText("发送");
        int ret = WCHIoCHubManager.getInstance().sessionClose(openSessionFd);
        if (ret == 0){
            targetNodeID.setText("");
            targetSessionKey.setText("");
            targetNodeID.setEnabled(true);
            targetSessionKey.setEnabled(true);
            openSessionBtn.setText("打开会话");
            stopSpeedTimer();
            setIOViewEnable(false);
        } else {
            showToast("关闭会话出错:" + ret);
        }
    }

    //读取数据
    private void sessionRead(int fd, int readLen){
        byte[] buffer = new byte[readLen];
        try {
            int len = WCHIoCHubManager.getInstance().sessionRead(fd,buffer,readLen);
            if (len != readLen){
                return;
            }
            receiveSize += len;
            sendMessage(SESSION_READ,"session_read",bytesToHexString(buffer));
        } catch (IoCHubException e) {
            showToast("读取失败:" + e.getErrorCode());
        }
    }

    //写入数据
    private void sessionWrite(){
        if (sendDataEdit.getText().length() == 0){
            showToast("未输入待发送数据");
            return;
        }
        String data = sendDataEdit.getText().toString();
        if (!autoSendCheckBox.isChecked()){
            try {
                //sessionWrite之前可先获取发送缓冲区可写长度sessionWriteableLen
                int len = WCHIoCHubManager.getInstance().sessionWrite(openSessionFd,data.getBytes());
                sendSize += len;
            } catch (IoCHubException e) {
                showToast("发送失败:" + e.getErrorCode());
            }
        } else {
            if (intervalEdit.getText().length() == 0){
                showToast("未输入发送间隔");
                return;
            }
            int val = Integer.parseInt(intervalEdit.getText().toString());
            startAutoSendTimer(data.getBytes(),val);
            sendBtn.setText("停止");
        }
    }

    //开启间隔发送定时器
    private void startAutoSendTimer(byte[] data,int interval){
        if (autoTimer == null){
            autoTimer = new Timer();
        }
        if (autoTimerTask == null){
            autoTimerTask = new TimerTask() {
                @Override
                public void run() {
                    try {
                        //sessionWrite之前可先获取发送缓冲区可写长度sessionWriteableLen
                        int len = WCHIoCHubManager.getInstance().sessionWrite(openSessionFd,data);
                        sendSize += len;
                    } catch (IoCHubException e) {
                        if (e.getErrorCode() != 401){
                            showToast("发送失败:" + e.getErrorCode());
                            stopAutoSendTimer();
                        }
                    }
                }
            };
        }
        if (!isAuto && autoTimer != null && autoTimerTask != null){
            isAuto = true;
            autoTimer.schedule(autoTimerTask,0,interval);
        }
    }

    //关闭间隔发送定时器
    private void stopAutoSendTimer(){
        isAuto = false;
        if (autoTimer != null){
            autoTimer.cancel();
            autoTimer = null;
        }
        if (autoTimerTask != null){
            autoTimerTask.cancel();
            autoTimerTask = null;
        }
    }

    //开启计数、速度定时器
    private void startSpeedTimer(){
        if (speedTimer == null){
            speedTimer = new Timer();
        }
        if (speedTimerTask == null){
            speedTimerTask = new TimerTask() {
                @Override
                public void run() {
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            recvSizeText.setText("接收:" + receiveSize + "B");
                            sendSizeText.setText("发送:" + sendSize + "B");
                            recvSpeedText.setText("速度:" + getSpeed(0) + "B/s");
                            sendSpeedText.setText("速度:" + getSpeed(1) + "B/s");
                        }
                    });
                }
            };
        }
        if (!isSpeed && speedTimer != null && speedTimerTask != null){
            isSpeed = true;
            speedTimer.schedule(speedTimerTask,0,1000);
        }
    }

    private long getSpeed(int state){
        long speed;
        if (state == 0){  //recv
            if (receiveSize == 0){
                copyRecvSize = 0;
                return 0;
            }
            if (copyRecvSize == 0){
                speed = receiveSize;
            } else {
                speed = receiveSize - copyRecvSize;
            }
            copyRecvSize = receiveSize;
        } else {
            if (sendSize == 0){
                copySendSize = 0;
                return 0;
            }
            if (copySendSize == 0){
                speed = sendSize;
            } else {
                speed = sendSize - copySendSize;
            }
            copySendSize = sendSize;
        }
        return speed;
    }

    //关闭计数、速度定时器
    private void stopSpeedTimer(){
        isSpeed = false;
        if (speedTimer != null){
            speedTimer.cancel();
            speedTimer = null;
        }
        if (speedTimerTask != null){
            speedTimerTask.cancel();
            speedTimerTask = null;
        }
    }

    private void sendMessage(int what){
        Message msg = new Message();
        msg.what = what;
        myHandler.sendMessage(msg);
    }

    private void sendMessage(int what, String key, String value){
        Message msg = new Message();
        msg.what = what;
        Bundle bundle = new Bundle();
        bundle.putString(key,value);
        msg.setData(bundle);
        myHandler.sendMessage(msg);
    }

    private Handler myHandler = new Handler(){
        @Override
        public void handleMessage(@NonNull Message msg) {
            super.handleMessage(msg);
            doHandler(msg);
        }
    };

    private void doHandler(Message msg) {
        switch (msg.what) {
            case START_SUCCESS:
                isStart = true;
                setMenu();
                getLocalNodeID();
                break;
            case LAN_SCAN_SUCCESS:
                lanDeviceSpinner.setAdapter(lanDeviceAdapter);
                break;
            case OPEN_SESSION_SUCCESS:
                startSpeedTimer();
                targetNodeID.setText(openNodeID);
                targetNodeID.setEnabled(false);
                targetSessionKey.setEnabled(false);
                openSessionBtn.setText("关闭会话");
                setIOViewEnable(true);
                break;
            case SESSION_CLOSED:
                stopSpeedTimer();
                stopAutoSendTimer();
                targetNodeID.setText("");
                targetSessionKey.setText("");
                targetNodeID.setEnabled(true);
                targetSessionKey.setEnabled(true);
                sendBtn.setText("发送");
                openSessionBtn.setText("打开会话");
                setIOViewEnable(false);
                break;
            case SESSION_READ:
                if (recvShowCheckBox.isChecked()){
                    Bundle receiveBundle = msg.getData();
                    String receiveMsg = receiveBundle.getString("session_read");
                    recvDataText.append(receiveMsg + " ");
                    recvDataTextRoll();
                }
                break;
        }
    }

    private void recvDataTextRoll(){
        int offset = recvDataText.getLineCount() * recvDataText.getLineHeight();
        int height = recvDataText.getHeight();
        if (offset > height) {
            recvDataText.scrollTo(0, offset - recvDataText.getHeight() + recvDataText.getLineHeight());
        }
    }

    private void setIOViewEnable(boolean enable){
        recvShowCheckBox.setEnabled(enable);
        autoSendCheckBox.setEnabled(enable);
        recvClearBtn.setEnabled(enable);
        sendClearBtn.setEnabled(enable);
        sendBtn.setEnabled(enable);
    }

    private void showToast(String message){
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(MainActivity.this,message,Toast.LENGTH_SHORT).show();
            }
        });
    }

    private String bytesToHexString(byte[] bArr) {
        if (bArr == null || bArr.length==0)
            return "";
        StringBuffer sb = new StringBuffer(bArr.length);
        String sTmp;
        for (int i = 0; i < bArr.length; i++) {
            sTmp = Integer.toHexString(0xFF & bArr[i]);
            if (sTmp.length() < 2) {
                sb.append(0);
            }
            sb.append(sTmp.toUpperCase());
            if (i != (bArr.length - 1)){
                sb.append(" ");
            }
        }
        bArr = null;
        return sb.toString();
    }

    @Override
    protected void onStop() {
        WCHIoCHubManager.getInstance().deInit();
        super.onStop();



    }
}