/* 
 * IocHub Demo for Linux       
 *
 * Copyright (C) 2004-2023 Nanjing Qinheng Microelectronics Co., Ltd.
 * Web:      http://wch.cn
 * Author:   WCH <tech@wch.cn>
 * Version:  V1.1
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 */
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QComboBox>
#include <QDateTime>
#include <QTimer>
#include <QList>
#include <QFileDialog>
#include <QFileInfo>
#include <QLabel>

#include "lanscandev.h"
#include "iochub.h"

QT_BEGIN_NAMESPACE
namespace Ui { class MainWindow; }
QT_END_NAMESPACE

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

    static void broadcast_cb(LAN_SCAN_RESULT_T * pLanDevNodes, int nodeSize);
    static void iochub_event(int eventCode, void * eventData);

signals:
    void emit_scanned_dev(LAN_SCAN_RESULT_T * pLanDevNodes, int nodeSize);
    void emit_iochub_event(int eventCode, void * eventData);

private slots:
    void on_refresh_network_interface_Btn_pressed();

    void on_start_service_Btn_pressed();

    void on_open_session_Btn_pressed();

    void on_lan_broadcast_Btn_pressed();

    void updateLanScanCombox(LAN_SCAN_RESULT_T * pLanDevNodes, int nodeSize);
    void slot_iochub_event(int eventCode, void * eventData);

    void slot_sending_file();

    void on_lan_device_list_comboBox_currentIndexChanged(const QString &arg1);

    void on_send_area_generate_btn_clicked();

    void on_recved_area_clear_btn_clicked();

    void on_recved_stop_show_checkbox_stateChanged(int arg1);

    void on_send_area_clear_btn_clicked();
    // 定时发送
    void on_delayed_timer_sending();
    // 更新收发速度
    void refresh_ui_speed();

    void on_send_area_send_btn_clicked();

    void on_send_timer_checkbox_stateChanged(int arg1);

    void on_access_cfg_set_Btn_pressed();

    void on_deInit_Btn_pressed();

    void on_send_file_sel_checkbox_clicked();

    void on_recved_file_saved_checkbox_clicked();

    void on_cfg_set_Btn_clicked();

    void on_session_transfer_type_Btn_clicked();

    void on_get_nat_type_Btn_clicked();

    void on_send_area_file_trans_btn_clicked();

private:
    Ui::MainWindow *ui;
    QTimer *delayedSendingTimer;     // 发送定时器
    QTimer *refreshSpeedTimer;       // 速度刷新定时器
    bool startService = false;
    bool sessionOpenFlag = false;
    bool recvedStopShowFlag = false;  // 停止接收显示
    bool sendTimerEnableFlag = false; // 发送定时器
    bool deInitFlag = false;          // 释放标识
    bool isSendingFlag = false;

    int sessionFd = 0;

    qint64 sendSpeed = 0;
    qint64 recvSpeed = 0;
    qint64 sendBytesCount = 0;
    qint64 oldSendBytesCount = 0;
    qint64 recvBytesCount = 0;
    qint64 oldRecvBytesCount = 0;

    QFile *localFile = nullptr;  //要发送的文件
    qint64 fileTotalBytes;  //数据总大小
    qint64 fileBytesWritten;  //已经发送数据大小
    qint64 fileSendPerPackageSize;   //每次发送数据的大小
    QString fileName;         //待发送文件路径
    bool sendingFileFlag = false;  // 正在发送文件标识
    QTimer *sendFileSpeedTimer;       // 发送定时器

    QString selectedSavingFilePath;
    QFile *savingFile = nullptr;
    bool saveIntoFileFlag = false;


    static MainWindow* m_This;
    QLabel *statusBarTitleLabel = nullptr;

    void handlerSendData();
    QString randCharToStr();
    /*主动打开会话*/
    void activeOpenSession(int sessionFd);
    void unActiveOpenSession(int sessionFd, QByteArray nodeIdStr);
    void closeSession(int sessionFd);
    void unActiveCloseSession();
    void changeTransferType(int transferType);

    bool openFile();         // 选择发送文件
    bool selectSavingFile(); // 选择接收文件路径

    QString size_transfer(uint64_t size);
    void read_data(int sessionFd, int length);
    /**
     * @brief getOrSetGlobalCfg
     * @param type 1:get 2:set
     */
    void getOrSetGlobalCfg(int type);

};
#endif // MAINWINDOW_H
