//*******************************************
//**  Copyright  (C)  WCH  2004-2023       **
//**  Web:  http://wch.cn                  **
//*******************************************
//**  iochub libray for iOS/macOS          **
//*******************************************
//
// iocHub接口库 V1.1
// 南京沁恒微电子
// 运行环境: iOS/macOS
//
   

import Cocoa

class ViewController: NSViewController {
    
    let iochub = IOCHubLibrary.shareInstace()
    
    @IBOutlet weak var locationIDLabel: NSTextField!
    @IBOutlet weak var startButton: NSButton!
    
    @IBOutlet weak var receiveBufferTextField: NSTextField!
    @IBOutlet weak var sendBufferTextField: NSTextField!
    @IBOutlet weak var maxTextField: NSTextField!
    @IBOutlet weak var packageSizeTextField: NSTextField!
    @IBOutlet weak var windowSizeTextField: NSTextField!
    @IBOutlet weak var windowTimeTextField: NSTextField!
    
    @IBOutlet weak var statusTextField: NSTextField!
    @IBOutlet weak var compressButton: NSButton!
    @IBOutlet weak var deivceLockButton: NSButton!
    @IBOutlet weak var netTypeTextField: NSTextField!
    @IBOutlet weak var inputDeviceTextField: NSTextField!
    @IBOutlet weak var communicateTypeTextField: NSTextField!
    @IBOutlet weak var deviceComboBox: NSComboBox!
    @IBOutlet weak var openButton: NSButton!
    @IBOutlet weak var inputSecretTextFiled: NSTextField!
    
    @IBOutlet weak var receiveTextView: NSTextView!
    @IBOutlet var sendTextView: NSTextView!
    @IBOutlet weak var sendTimerTextFiled: NSTextField!
    
    @IBOutlet weak var receiveBytesLabel: NSTextField!
    @IBOutlet weak var receiveSpeedLabel: NSTextField!
    @IBOutlet weak var sendBytesLabel: NSTextField!
    @IBOutlet weak var sendSpeedLabel: NSTextField!
    @IBOutlet weak var sendButton: NSButton!
    @IBOutlet weak var sendFileButton: NSButton!
    
    @IBOutlet weak var dataFileButton: NSButton!
    @IBOutlet weak var saveFileTextField: NSTextField!
    @IBOutlet weak var dataFileTextField: NSTextField!
    @IBOutlet weak var compareFileButton: NSButton!
    @IBOutlet weak var compareFileTextField: NSTextField!
    
    private var deviceList:[String:String] = [:]
    private var sesssionList:[String:Int32] = [:]
    
    private var startStatus:Bool = false
    private var isFileSend:Bool = false
    
    private var isTimerSend:Bool = false
    private var showLogInfo = true
    //定时发送
    private var isSendWork:Bool = false
    //文件发送
    private var isSendFile:Bool = false
    //出错后停止保持文件和继续对比
    private var isStopCompare:Bool = false
    
    private var receiveFileSave = false
    private var receiveContentShow = false
    private var receiveAddTail = false
    private var receiveAutoScroll = false
    private var receiveFilePath:URL?
    private var receiveBlock:DispatchWorkItem?
    
    private var compareFileSelected = false
    private var compareFilePath:URL?
    private var compareFileHandle:FileHandle?
    private var compareFileStartIndex:UInt64 = 0
    private var compareFileEndIndex:UInt64 = 0
    
    private var sendFileSelected = false
    private var sendAddTail = false
    private var sendFilePath:URL?
    private var sendFileHandle:FileHandle?
    private var sendFileEndIndex:UInt64 = 0
    private var sendBlock:DispatchWorkItem?
    private var sendTimer:DispatchSourceTimer?
    
    private weak var updateTimer: Timer?
    var sendTimeGap:TimeInterval = 0
    
    private lazy var compareQueue:OperationQueue = {
        let queue = OperationQueue()
        queue.maxConcurrentOperationCount = 1
        return queue
    }()
    
    private var totalReceiveCount = 0
    private var prevRecevieCount = 0
    private var totalSendCount = 0
    private var prevSendCount = 0
    private var receiveClearCount = 0
    
    private var uploadSpeed: Double = 0.0
    private var downloadSpeed: Double = 0.0
    private var uploadMetric: String = "KB"
    private var downloadMetric: String = "KB"
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view.
        iochub?.delegate = self
        guard let model = iochub?.getConfig() else {return}
        self.addDefaultParams(model)
        
        self.addUpdateTimer()
    }
    
    @IBAction func startIochubAction(_ sender: Any) {
        if (self.startStatus) {
            iochub?.stop()
            self.startButton.title = "启动"
        }else {
            guard let model = iochub?.getConfig() else {return}
            self.addDefaultParams(model)
            iochub?.start("58.213.74.190:38089")
            self.startButton.title = "关闭"
        }
    }
    
    @IBAction func sureAction(_ sender: Any) {
        let model = IOCHubCfgModel.init()
        model.recvBufferSize = UInt32(self.receiveBufferTextField.stringValue) ?? 0
        model.sendBufferSize = UInt32(self.sendBufferTextField.stringValue) ?? 0
        model.maxConnSize = UInt32(self.maxTextField.stringValue) ?? 0
        model.sendSizePerPackage = UInt32(self.packageSizeTextField.stringValue) ?? 0
        model.socketWndSize = UInt32(self.windowSizeTextField.stringValue) ?? 0
        model.socketWndTimeout = UInt32(self.windowTimeTextField.stringValue) ?? 0
        
        if (self.compressButton.state == .on) {
            model.supportCompress = true
        }else {
            model.supportCompress = false
        }
        
        if (self.deivceLockButton.state == .on) {
            model.openDeviceLock = true
        }else {
            model.openDeviceLock = false
        }
        
        self.iochub?.setConfig(model)
    }
    
    private func addDefaultParams(_ model:IOCHubCfgModel) {
        self.receiveBufferTextField.stringValue = "\(model.recvBufferSize)"
        self.sendBufferTextField.stringValue = "\(model.sendBufferSize)"
        self.maxTextField.stringValue = "\(model.maxConnSize)"
        self.packageSizeTextField.stringValue = "\(model.sendSizePerPackage)"
        self.windowSizeTextField.stringValue = "\(model.socketWndSize)"
        self.windowTimeTextField.stringValue = "\(model.socketWndTimeout)"
        
        if (model.supportCompress) {
            self.compressButton.state = .on
        }else {
            self.compressButton.state = .off
        }
        
        if (model.openDeviceLock) {
            self.deivceLockButton.state = .on
        }else {
            self.deivceLockButton.state = .off
        }
    }
    
    @IBAction func lanScanAction(_ sender: Any) {
        self.deviceComboBox.removeAllItems()
        iochub?.scanLanNodes("", timeout: 1000, callBack: {[weak self] (address, nodeId, size) in
            guard let wself = self else {return}
            let ip_port:String = address ?? ""
            let nodeId:String = nodeId ?? ""
            if (nodeId.count > 0 && ip_port.count > 0) {
                wself.deviceList[nodeId] = ip_port
            }
            DispatchQueue.main.async {
                wself.deviceComboBox.addItem(withObjectValue: nodeId)
            }
        })
    }
    
    @IBAction func deviceSelectAction(_ sender: Any) {
        self.inputDeviceTextField.stringValue =  self.deviceComboBox.stringValue
    }
    
    
    func openWithDeviceID(_ deviceID:String, secret:String) {
        if (deviceID.count == 0) {
            print("设备ID不存在")
            return
        }
        iochub?.sessionOpen(deviceID, dstSessionKey: secret)
    }
    
    @IBAction func openAction(_ sender: Any) {
        
        let deviceID = self.inputDeviceTextField.stringValue
        let sessionFd = self.sesssionList[deviceID] ?? 0
        if (sessionFd == 0) {
            self.openWithDeviceID(self.inputDeviceTextField.stringValue, secret: self.inputSecretTextFiled.stringValue)
        }else {
            let ret = iochub?.sessionClose(sessionFd)
            if (ret == 0) {
                self.statusTextField.stringValue = "会话已关闭"
            }else {
                self.statusTextField.stringValue = "会话关闭出错\(ret ?? -1)"
            }
            
            self.openButton.title = "打开会话"
            self.sesssionList.removeValue(forKey: deviceID)
            self.inputDeviceTextField.stringValue = ""
        }
    }
    
    @IBAction func checkNetType(_ sender: Any) {
        let type = iochub?.getNATType()
        var typeStr = ""
        switch type{
        case 0x00:
            typeStr = "非对称类型"
            break
        case 0x01:
            typeStr = "全锥型"
            break
        case 0x02:
            typeStr = "IP受限锥型"
            break
        case 0x03:
            typeStr = "端口受限锥型"
            break
        case 0x04:
            typeStr = "具有固定IP地址"
            break
        case 0x10:
            typeStr = "对称NAT"
            break
        case 0x11:
            typeStr = "不支持UDP"
            break
        case 0xFF:
            typeStr = "未知类型"
            break
        default:
            typeStr = "获取网络类型失败"
            break
        }
        self.netTypeTextField.stringValue = typeStr
    }
    
    @IBAction func checkCommunicateType(_ sender: Any) {
        
        let deviceID = self.inputDeviceTextField.stringValue
        let sessionFd = self.sesssionList[deviceID] ?? 0

        let type = iochub?.getTransferType(Int32(sessionFd))
        var typeStr = ""
        
        switch type {
        case 0x00:
            typeStr = "未知类型"
            break
        case 0x01:
            typeStr = "P2P穿透"
            break
        case 0x02:
            typeStr = "服务转发"
            break
        case 0x03:
            typeStr = "局域网"
            break
        default:
            typeStr = "获取通讯类型失败"
            break
        }
        self.communicateTypeTextField.stringValue = typeStr
    }
    
    @IBAction func sendDataAction(_ sender: Any) {
        let text = self.sendTextView.string
        if (text.count == 0) {
            return
        }
        
        guard let data = text.data(using: .utf8) else {return}
        if (self.isTimerSend) {
            if (self.isSendWork) {
                self.stopTimer()
            }else {
                let sendtime = (Double(self.sendTimerTextFiled.stringValue) ?? 0) / 1000
                if (sendtime <= 0) {
                    return
                }
                self.startTimer(data, time: sendtime)
            }
        }else {
            let deviceID = self.inputDeviceTextField.stringValue
            let sessionFd = self.sesssionList[deviceID] ?? 0
            self.sendContent(sessionFd: Int32(sessionFd), data: data)
        }
    }
    
    private func socketSendFile() {
        guard let sendFileHandle = self.sendFileHandle else { return }
     
        do { try sendFileHandle.seek(toOffset: 0) }catch{ return }
        
        self.isSendFile = true
        var offsetBy:Int = Int(self.packageSizeTextField.stringValue) ?? 0
        let deviceID = self.inputDeviceTextField.stringValue
        let sessionFd = self.sesssionList[deviceID] ?? 0
        
        DispatchQueue.global().async { [weak self] in
            guard let wself = self else { return }
            var startFileIndex:UInt64 = 0
            let endFileIndex = wself.sendFileEndIndex
            while startFileIndex < endFileIndex {
                //确保文件传输停止，防止对象无法正常释放
                if (!wself.isSendFile) {
                    break
                }
                
                autoreleasepool {
                    do {
                        let sendData = try sendFileHandle.read(upToCount: offsetBy)
                        wself.sendContent(sessionFd: Int32(sessionFd), data: sendData!)
                        startFileIndex = try sendFileHandle.offset()
                        if (startFileIndex == endFileIndex) {
                            try sendFileHandle.seek(toOffset: 0)
                        }
//                        Thread.sleep(forTimeInterval: 0.001)
                    }catch {
                        
                    }
                }
            }
            
            wself.isSendFile = false
            
            DispatchQueue.main.async {
                wself.sendFileButton.title = "发送文件"
            }
        }
    }
    
    func sendContent(sessionFd:Int32, data:Data) {
        if (data.count == 0) {
            return
        }
        
        self.writeDataToSocket(sessionFd: sessionFd, data: data)
    }
    
    func writeDataToSocket(sessionFd:Int32, data:Data) {
    
        let count = data.count
        if (count == 0 || sessionFd == 0) {
            return
        }
        var length:Int = Int(self.iochub?.sessionCouldWriteableLen(sessionFd) ?? 0)
        while length < count {
            length = Int(self.iochub?.sessionCouldWriteableLen(sessionFd) ?? 0)
            Thread.sleep(forTimeInterval: 0.001)
        }
                
        let writeLen = self.iochub?.sessionWrite(sessionFd, write: data) ?? 0
        if (writeLen > 0) {
            self.totalSendCount += Int(writeLen)
        }
//        //5k数据
//        if (data.count == 0 || sessionFd == 0) {
//            return
//        }
//        autoreleasepool {
//            var sendData = Data.init(data)
//            let count = data.count
//
//            let length:Int = Int(self.iochub?.sessionWrite(sessionFd, write: sendData) ?? 0)
//            //3K
//            if (length < 0) {
//
//                if (length != -401) {
//                    print("errorCode:\(length)", length)
//                    return
//                }
//
//                Thread.sleep(forTimeInterval: 0.005)
//                self.writeDataToSocket(sessionFd: sessionFd, data: sendData)
//            }else {
//
//                self.totalSendCount += Int(length)
//
//                if (length < count) {
//                    let sliceData = Data.init(data)
//                    sendData = sliceData [length ..< count]
//                }else {
//                    sendData = Data.init()
//                }
//                Thread.sleep(forTimeInterval: 0.001)
//                //
//                self.writeDataToSocket(sessionFd: sessionFd, data: sendData)
//            }
//        }
    }
    
    private func startTimer(_ data:Data, time: TimeInterval) {
        if (self.isSendWork) {
            return
        }
        
        if (self.sendTimer == nil) {
            self.sendTimer = DispatchSource.makeTimerSource(flags: [], queue: DispatchQueue.global())
        }
    
        let deviceID = self.inputDeviceTextField.stringValue
        let sessionFd = self.sesssionList[deviceID] ?? 0
        
        self.sendTimer?.schedule(deadline: .now(), repeating: time, leeway: DispatchTimeInterval.seconds(0))
        self.sendTimer?.setEventHandler { [weak self] in
            guard let wself = self else { return }
            
            wself.sendContent(sessionFd: Int32(sessionFd), data: data)
        }
        
        self.isSendWork = true
        self.sendButton.title = "停止发送"
        self.sendTimer?.resume()
    }
    
    private func stopTimer() {
        self.isSendWork = false
        self.sendButton.title = "循环发送"
        
        if (self.sendTimer != nil) {
            self.sendTimer?.cancel()
            self.sendTimer = nil
        }
    }
    
    @IBAction func sendTimerAction(_ sender: Any) {
        self.isTimerSend = !self.isTimerSend
        self.sendTimerTextFiled.isEnabled = self.isTimerSend
        
        if (self.isTimerSend) {
            self.sendButton.title = "循环发送"
        }else {
            self.sendButton.title = "发送"
        }
    }
    
    @IBAction func clearTextAction(_ sender: NSButton) {
        let tag = sender.tag
        if (tag == 1) {
            self.receiveTextView.string = ""
        }else if (tag == 2) {
            self.sendTextView.string = ""
        }else if (tag == 3) {
           
        }else if (tag == 4) {
            self.receiveBytesLabel.stringValue = "0"
            self.receiveSpeedLabel.stringValue = "0KB/s"
            self.sendBytesLabel.stringValue = "0"
            self.sendSpeedLabel.stringValue = "0KB/s"
            self.totalSendCount = 0
            self.totalReceiveCount = 0
            self.prevSendCount = 0
            self.prevRecevieCount = 0
        }
    }
    
    @IBAction func sendFileAction(_ sender: Any) {
        if (self.sendFileHandle == nil) {
            return
        }
        
        if (!self.isSendFile) {
            self.socketSendFile()
            self.sendFileButton.title = "停止发送"
       
        }else {
            self.isSendFile = false
            self.sendFileButton.title = "发送文件"
        }
    }
    
    @IBAction func saveFileAction(_ sender: Any) {
        
        let saveBtn = sender as! NSButton
        self.receiveFileSave = !self.receiveFileSave
        if (!self.receiveFileSave) {
            self.receiveFilePath = nil
            self.saveFileTextField.stringValue = NSLocalizedString("contentview.unselected.file", comment: "")
            return
        }
        
        let openPanel = NSOpenPanel()       // Authorize access in sandboxed mode
        openPanel.message = NSLocalizedString("Select folder where to create file\n(Necessary to manage security on this computer)", comment: "enableFileMenuItems")
        openPanel.prompt = NSLocalizedString("Select", comment: "enableFileMenuItems")
        openPanel.canChooseFiles = false    // Only select or create Directory here ; you can select the real Desktop
        openPanel.canChooseDirectories = true
        openPanel.canCreateDirectories = true
        openPanel.begin() {[weak self]                             // In the completion, Save the file
            (result2) -> Void in
            guard let wself = self else {return}
            if result2 == NSApplication.ModalResponse.OK {
                let savePanel = NSSavePanel()
                savePanel.title = NSLocalizedString("File to create", comment: "enableFileMenuItems")
                savePanel.nameFieldStringValue = "WCH"
                savePanel.prompt = NSLocalizedString("Create", comment: "enableFileMenuItems")
                savePanel.allowedFileTypes = ["txt"]   // if you want to specify file signature
                let fileManager = FileManager.default
                
                savePanel.begin() { (result) -> Void in
                    
                    if result == NSApplication.ModalResponse.OK {
                        let fileWithExtensionURL = savePanel.url!  //  May test that file does not exist already
                        wself.receiveFilePath = fileWithExtensionURL
                        wself.saveFileTextField.stringValue = fileWithExtensionURL.path
                        let exist = fileManager.fileExists(atPath: fileWithExtensionURL.path)
                        if !exist {
                            fileManager.createFile(atPath: fileWithExtensionURL.path,contents:nil,attributes:nil)
                            saveBtn.state = .on
                            wself.receiveFileSave = true
                        }
                    }else {
                        saveBtn.state = .off
                        wself.receiveFileSave = false
                    }
                }
            }else {
                saveBtn.state = .off
                wself.receiveFileSave = false
            }
        }
    }
    
    
    @IBAction func selectFileAction(_ sender: NSButton) {
        
        let tag = sender.tag
        if (tag == 1 && self.sendFileSelected) {
            self.sendFileSelected = false
            self.dataFileButton.state = .off
            self.dataFileTextField.stringValue = NSLocalizedString("未选择文件", comment: "")
            
            self.sendFileHandle?.closeFile()
            self.sendFileHandle = nil
            self.sendFilePath = nil
            return
        }else if tag == 2 && self.compareFileSelected {
            self.compareFileSelected = false
            self.isStopCompare = false
            self.compareFileButton.state = .off
            self.compareFileTextField.stringValue = NSLocalizedString("未选择文件", comment: "")
            
            self.compareFileHandle?.closeFile()
            self.compareFileHandle = nil
            self.compareFilePath = nil
            return
        }
        
        let panel = NSOpenPanel()
        panel.canChooseFiles = true
        panel.canChooseDirectories = true
        panel.canCreateDirectories = false
        panel.allowsMultipleSelection = false
        
        guard let window = NSApplication.shared.mainWindow else { return }
        
        panel.beginSheetModal(for: window) {[weak self] (result) in
            guard let wself = self else { return }
            if result == .OK {
                guard let selectedPath = panel.url else {return}
                if tag == 1 {
              
                    wself.dataFileButton.state = .on
                    wself.dataFileTextField.stringValue = selectedPath.path
                    
                    wself.sendFileSelected = true
                    wself.sendFilePath = selectedPath
                
                    let fileHandle = try? FileHandle.init(forReadingFrom: selectedPath)
                    wself.sendFileEndIndex = fileHandle?.seekToEndOfFile() ?? 0
                    fileHandle?.closeFile()
                    
                    wself.sendFileHandle?.closeFile()
                    wself.sendFileHandle = try? FileHandle.init(forReadingFrom: selectedPath)
                    
                }else if tag == 2 {
                    
                    wself.compareFileButton.state = .on
                    wself.compareFileTextField.stringValue = selectedPath.path
                    
                    wself.isStopCompare = false
                    wself.compareFileSelected = true
                    wself.compareFilePath = selectedPath
                   
                    let fileHandle = try? FileHandle.init(forReadingFrom: selectedPath)
                    wself.compareFileEndIndex = fileHandle?.seekToEndOfFile() ?? 0
                    fileHandle?.closeFile()
                    
                    wself.compareFileHandle?.closeFile()
                    wself.compareFileHandle = try? FileHandle.init(forReadingFrom: selectedPath)
                }

            }else {
                
                panel.close()
                
                if tag == 1 {
                    wself.sendFileSelected = false
                    wself.dataFileButton.state = .off
   
                    wself.sendFileHandle?.closeFile()
                    wself.sendFileHandle = nil
                    wself.sendFilePath = nil
                }else if tag == 2 {
                    wself.compareFileSelected = false
                    wself.compareFileButton.state = .off
                    
                    wself.compareFileHandle?.closeFile()
                    wself.compareFileHandle = nil
                    wself.compareFilePath = nil
                }
            }
        }
    }
    
    func  addUpdateTimer() {
        
        self.updateTimer = Timer.scheduledTimer(withTimeInterval: 1.0, repeats: true) { _ in
            
            if (self.totalReceiveCount == 0) {
                self.downloadSpeed = 0.0
                self.downloadMetric = "KB"
            }else {
                let currentcount = self.totalReceiveCount
                let deltain: Double = Double(currentcount - self.prevRecevieCount) / 1024.0
                self.prevRecevieCount = currentcount
                
                if (deltain > 1000.0) {
                    self.downloadSpeed = deltain / 1024.0
                    self.downloadMetric = "MB"
                } else {
                    self.downloadSpeed = deltain
                    self.downloadMetric = "KB"
                }
                
                DispatchQueue.main.async {
                    self.receiveBytesLabel.stringValue = "\(self.totalReceiveCount)"
                    self.receiveSpeedLabel.stringValue = String.init(format: "%0.1f%@/s", self.downloadSpeed, self.downloadMetric)
                }
                
            }
            
            if (self.totalSendCount == 0) {
                self.uploadSpeed = 0.0
                self.uploadMetric = "KB"
            }else {
                let currentcount = self.totalSendCount
                let deltaout: Double = Double(currentcount - self.prevSendCount) / 1024.0
                self.prevSendCount = currentcount
                
                if (deltaout > 1000.0) {
                    self.uploadSpeed = deltaout / 1024.0
                    self.uploadMetric = "MB"
                } else {
                    self.uploadSpeed = deltaout
                    self.uploadMetric = "KB"
                }
                
                DispatchQueue.main.async {
                    self.sendBytesLabel.stringValue = "\(self.totalSendCount)"
                    self.sendSpeedLabel.stringValue = String.init(format: "%0.1f%@/s", self.uploadSpeed, self.uploadMetric)
                }
                
            }
        }
        
        RunLoop.current.add(self.updateTimer!, forMode: .common)
    }
    

}

extension ViewController:IOCHubLibraryDelegate {
    func iochubStart(_ status: Int) {
        print("iochubStart status:\(status)");
        self.startStatus = true
        DispatchQueue.main.async {
            self.locationIDLabel.stringValue = "设备ID：" +  (self.iochub?.getLocalID() ?? "")
        }
    }
    
    func iochubStop(_ status: Int) {
        print("iochubStop status:\(status)");
        self.startStatus = false
        DispatchQueue.main.async {
            self.locationIDLabel.stringValue = "设备ID："
        }
    }
    
    func iochubActiveSessionOpen(_ sessionID: Int, deviceID: String?, status: Int) {
        print("iochubActiveSessionOpen sessionID:\(sessionID) deviceID:\(String(describing: deviceID)) status:\(status)");
        DispatchQueue.main.async {
            if (status == 0) {
                if (sessionID > 0) {
                    guard let device = deviceID else {return}
                    self.sesssionList[device] = Int32(sessionID)
                    self.inputDeviceTextField.stringValue = device
                    self.statusTextField.stringValue = "会话打开成功"
                    self.openButton.title = "关闭会话"
                }else {
                    self.statusTextField.stringValue = "打开失败:\(status)"
                    self.openButton.title = "打开会话"
                }
            }else {
                self.statusTextField.stringValue = "打开失败:\(status)"
                self.openButton.title = "打开会话"
            }
        }
    }
    
    func iochubUnactiveSessionOpen(_ sessionID: Int, deviceID: String?, status: Int) {
        print("iochubUnactiveSessionOpen sessionID:\(sessionID) deviceID:\(String(describing: deviceID)) status:\(status)");
        DispatchQueue.main.async {
            if (status == 0) {
                if (sessionID > 0) {
                    guard let device = deviceID else {return}
                    self.sesssionList[device] = Int32(sessionID)
                    self.inputDeviceTextField.stringValue = device
                    
                    self.statusTextField.stringValue = "会话打开成功"
                    self.openButton.title = "关闭会话"
                }else {
                    self.statusTextField.stringValue = "打开失败:\(status)"
                    self.openButton.title = "打开会话"
                }
            }else {
                self.statusTextField.stringValue = "打开失败:\(status)"
                self.openButton.title = "打开会话"
            }
        }
    }
    
    func iochubSessionClose(_ sessionID: Int, deviceID: String?, status: Int) {
        print("iochubSessionClose sessionID:\(sessionID) deviceID:\(String(describing: deviceID)) status:\(status)");
        DispatchQueue.main.async {
            if (status == 0) {
                if (sessionID > 0) {
                    guard let device = deviceID else {return}
                    self.sesssionList.removeValue(forKey: device)
                    self.inputDeviceTextField.stringValue = ""
                  
                    self.openButton.title = "打开会话"
                }
            }else {
                self.openButton.title = "打开会话"
            }
        }
    }
    
    func iochubSessionRead(_ sessionID: Int) {
        autoreleasepool {
            guard let data = self.iochub?.sessionRead(Int32(sessionID), length: 65535) else {return}
            self.totalReceiveCount += data.count
            self.compareReceiveData(data)
            DispatchQueue.main.async {
                let dataStr = String.init(data: data, encoding: .utf8) ?? ""
                let textStr = NSAttributedString.init(string: dataStr)
                print(textStr)
//                self.receiveTextView.textStorage?.append(textStr)
            }
        }
    }
    
    func iochubSessionTransferTypeChange(_ sessionID: Int, type: Int) {
        let errorStr = "iochubSessionTransferTypeChange sessionID:\(sessionID) type:\(type)"
        print(errorStr)
    }
    
    func iochubSessionException(_ sessionID: Int, status: Int) {
        let errorStr = "iochubSessionException sessionID:\(sessionID) status:\(status)"
        print(errorStr)
    }
    
    func iochubOperationErr(_ status: Int) {
        let errorStr = "iochubOperationErr status:\(status)"
        print(errorStr)
    }
    
    func iochubNetworkChange(_ status: Bool, type: Int) {
        if (status) {
            if (type == 1) {
              
            }else {
                
            }
        }else {
           
        }
    }
    
    private func compareReceiveData(_ data:Data) {
        
        if (!self.compareFileSelected || self.isStopCompare) {
            return
        }
        
        do {
            guard let compareFileHandle = self.compareFileHandle else {return}
            var currentData = data
                       
            guard var compareData = try compareFileHandle.read(upToCount: data.count) else {return}
            if (data.count > (compareData.count)) {
                currentData = data[0..<compareData.count]
            }
            
            //数据出错处理
            if (compareData != currentData && compareData.count != 0) {
                self.isStopCompare = true
                self.receiveErrorInfo(currentData, compareData: compareData)
            }
            self.compareFileStartIndex = try compareFileHandle.offset()
            
            if (self.compareFileStartIndex == self.compareFileEndIndex) {
                try compareFileHandle.seek(toOffset: 0)
                let seporateCount = data.count - compareData.count
                if (seporateCount > 0) {
                    currentData = data[compareData.count..<data.count]
                    try compareData =  compareFileHandle.read(upToCount: seporateCount) ?? Data()
                }
                
                if (compareData != currentData && compareData.count != 0) {
                    self.isStopCompare = true
                    self.receiveErrorInfo(currentData, compareData: compareData)
                }
                self.compareFileStartIndex = try compareFileHandle.offset()
            }
            
        }catch {
            
        }
    }
    
    private func receiveErrorInfo(_ currentData:Data, compareData:Data) {
        var errorStr:String = ""
        for index in 0 ..< currentData.count {
            if (currentData[index] != compareData[index]) {
                let str = String.init(format: "[%d][receive:0x%2x - local:0x%2x] ",(self.compareFileStartIndex + UInt64(index)), currentData[index], compareData[index])
                errorStr += str
                break
            }
        }
        
        if (self.isStopCompare) {
            self.compareFileSelected = false
            self.compareFileStartIndex = 0
            DispatchQueue.main.async {
                self.compareFileButton.state = .off
            }
        }
        
        print(errorStr)
    }
}
