//*******************************************
//**  Copyright  (C)  WCH  2004-2023       **
//**  Web:  http://wch.cn                  **
//*******************************************
//**  iochub libray for iOS/macOS          **
//*******************************************
//
// iocHub接口库 V1.1
// 南京沁恒微电子
// 运行环境: iOS/macOS
//

import UIKit
import MBProgressHUD

open class MH_VC_BaseViewController: UIViewController {
    
    public enum NavbarStatusType {
        case Show       //显示，默认
        case Hide       //隐藏
        case Custom     //自定义
    }
    
    open var navbarStatus: NavbarStatusType = .Show
    
   // private var hud: MBProgressHUD?    
    deinit{
        NotificationCenter.default.removeObserver(self)
    }
    open func predeinit() {
    }
    
    public override init(nibName nibNameOrNil: String?, bundle nibBundleOrNil: Bundle?) {
        super.init(nibName: nibNameOrNil, bundle: nibBundleOrNil);
        self.hidesBottomBarWhenPushed = true;
    }
    
    public required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder);
    }

    
    open override func viewDidLoad() {
        super.viewDidLoad()

        self.view.backgroundColor = UIColor.white
        self.edgesForExtendedLayout = [];
//        self.automaticallyAdjustsScrollViewInsets = false
        self.extendedLayoutIncludesOpaqueBars = true
        
        //暂时关闭ios13的Dark Mode
        if #available(iOS 13.0, *) {
            self.overrideUserInterfaceStyle = .light
        }
        
    }
    
    open override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        self.setNeedsStatusBarAppearanceUpdate()
        if self.navbarStatus == .Show {
            self.navigationController?.setNavigationBarHidden(false, animated: true)
        }else if self.navbarStatus == .Hide {
            self.navigationController?.setNavigationBarHidden(true, animated: true)
        }
//        let page = String(describing: type(of: self))
  
    }
    
    open override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
//        let page = String(describing: type(of: self))

    }
    //MARK: public
    public func rightBarItem(with title: String, selector: Selector?) -> UIBarButtonItem {
        let rightItem = UIBarButtonItem.init(title: title, style: .plain, target: self, action: selector)
        rightItem.setTitleTextAttributes([NSAttributedString.Key.font : UIFont.systemFont(ofSize: 16.0)], for: .normal)
        self.navigationItem.rightBarButtonItems = [rightItem]
        
        return rightItem
    }
    public func rightBarItem(with image: UIImage?, selector: Selector?) -> UIBarButtonItem {
        let rightItem = UIBarButtonItem.init(image: image, style: .plain, target: self, action: selector)
        self.navigationItem.rightBarButtonItems = [rightItem]
        
        return rightItem
    }

    public func leftBarItem(with title: String, selector: Selector?) -> UIBarButtonItem {
        let leftItem = UIBarButtonItem.init(title: title, style: .plain, target: self, action: selector ?? #selector(self.backAction))
        leftItem.setTitleTextAttributes([NSAttributedString.Key.font:UIFont.systemFont(ofSize: 16.0)], for: .normal)
        self.navigationItem.leftBarButtonItems = [leftItem]
        
        return leftItem
    }
    
    public func leftBarItem(with image: UIImage?, selector: Selector?) -> UIBarButtonItem {
        let leftItem = UIBarButtonItem.init(image: image, style: .plain, target: self, action: selector ?? #selector(self.backAction))
        self.navigationItem.leftBarButtonItems = [leftItem]
        
        return leftItem
    }
    
    @objc open func backAction() {
        if self.navigationController != nil && (self.navigationController?.viewControllers.count)! > Int(1) {
            self.navigationController?.popViewController(animated: true)

            return
        }else if (self.presentingViewController != nil) {
            self.presentingViewController?.dismiss(animated: true, completion: nil)

            return
        }

        self.navigationController?.popViewController(animated: true)
    }
    
    
    public func hideToast() {
        
        MBProgressHUD.hide(for: self.view, animated: false)
        
        //self.hud?.removeFromSuperview()
        //self.hud = nil
    }
    
    open override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
        

    
}

extension UIViewController {
    
    public func showHud(_ text:String) -> MBProgressHUD {
        
        hideHud()
        
        let hud = MBProgressHUD.showAdded(to: self.view, animated: true)
        hud.mode = .text
        hud.label.text = text
        //mbHud = hud
        return hud
        
    }
    
    public func showDetailHud(_ text:String) -> MBProgressHUD {
        
        hideHud()
        
        let hud = MBProgressHUD.showAdded(to: self.view, animated: true)
        hud.mode = .text
        hud.detailsLabel.text = text
        //mbHud = hud
        return hud
        
    }
    
    public func showLoadingWithText(_ text:String?) -> MBProgressHUD{
        
        hideHud()
        var message = text
        if nil == message {
            message = "Loading..."
        }
        
        let hud = showHud(message!)
        hud.mode = .indeterminate
        return hud
        
    }
    
    public func showLoadingDetailText(_ text:String?) -> MBProgressHUD{
        
        hideHud()
        var message = text
        if nil == message {
            message = "Loading..."
        }
        
        let hud = showDetailHud(message!)
        hud.mode = .indeterminate
        return hud
        
    }
    
    
    public  func showProcessingHud() -> MBProgressHUD {
        
        hideHud()
        
        let hud = MBProgressHUD.showAdded(to: self.view, animated: true)
        hud.mode = .indeterminate
        //mbHud = hud
        return hud
    }
    
    public func hideHud() {
        /*
         if nil != mbHud {
         mbHud!.hide(animated: false)
         mbHud = nil
         }
         */
        
        MBProgressHUD.hide(for: self.view, animated: false)
        
    }
    
    public func showSuccessMessage(_ text: String,long:Bool = false ) ->MBProgressHUD {
        
        hideHud()
        
        let hud = MBProgressHUD.showAdded(to: self.view, animated: true)
        
        
        hud.mode = .customView
        // Set an image view with a checkmark.
        let image = UIImage(named: "default_hud_success")?.withRenderingMode(.alwaysTemplate)
        
        let customView = UIImageView(image: image)
        hud.customView = customView
        // Looks a bit nicer if we make it square.
        hud.isSquare = false
        // Optional label text.
        hud.label.text = text
        if long {
            hud.hide(animated: true, afterDelay: 2)

        }else {
            hud.hide(animated: true, afterDelay: 0.5)

        }
        
        return hud
    }
    
    public func showFailedMessage(_ text: String) -> MBProgressHUD {
        
        hideHud()
        
        let hud = MBProgressHUD.showAdded(to: self.view, animated: true)
        
        
        hud.mode = .customView
        // Set an image view with a checkmark.
        let image = UIImage(named: "default_hud_error")?.withRenderingMode(.alwaysTemplate)
        let customView = UIImageView(image: image)
        hud.customView = customView
        // Looks a bit nicer if we make it square.
        hud.isSquare = false
        // Optional label text.
        hud.detailsLabel.text = text
        //hud.detailsLabel.text = text
        hud.hide(animated: true, afterDelay: 1.5)
        
        return hud
    }
    
    
    public func showLongFailedMessage(_ text: String) -> MBProgressHUD {
        
        hideHud()
        
        let hud = MBProgressHUD.showAdded(to: self.view, animated: true)
        
        
        hud.mode = .customView
        // Set an image view with a checkmark.
        let image = UIImage(named: "default_hud_error")?.withRenderingMode(.alwaysTemplate)
        let customView = UIImageView(image: image)
        hud.customView = customView
        // Looks a bit nicer if we make it square.
        hud.isSquare = false
        // Optional label text.
        hud.detailsLabel.text = text
        hud.hide(animated: true, afterDelay: 2.0)
        
        return hud
    }
    
    //Toast LONG_DELAY = 3500; // 3.5 seconds
    //SHORT_DELAY = 2000; // 2 seconds
    public func showQuickTipMessage(_ text: String) -> MBProgressHUD {
        
        hideHud()
        
        let hud = MBProgressHUD.showAdded(to: self.view, animated: true)
        
        let size = UIScreen.main.bounds.size
        hud.mode = .text
        hud.offset = CGPoint(x: 0, y: size.height/2 + 60)
        
        // Looks a bit nicer if we make it square.
        hud.isSquare = false
        // Optional label text.
        //hud.label.text = text
        hud.detailsLabel.text = text
        hud.hide(animated: true, afterDelay: 0.7)
        
        return hud
    }
    
    public func showAlert(title:String , message:String, okTitle:String,okhandler:((UIAlertAction) -> Void)? = nil) -> UIAlertController {
        let alert = UIAlertController(title: title, message: message, preferredStyle: .alert)
        
        alert.addAction(UIAlertAction(title:okTitle, style: .default, handler: okhandler))
        alert.addAction(UIAlertAction(title: "取消", style: .cancel, handler: nil))
        
        self.present(alert, animated: true)
        
        return alert
    }
}


public extension UIView {
    
    class func instanceFromNib<T:UIView>(_ viewType:T.Type) -> T? {
        let bundle = Bundle.init(for: viewType)
        let nibs = bundle.loadNibNamed(String(describing: viewType), owner: nil, options: nil)
        return nibs?.first as? T
    }
    
    
    class func instanceFromNib() -> Self? {
        return instanceFromNib(self)
    }
    
    func getFirstViewController()-> UIViewController? {
        
        for view in sequence(first: self.superview, next: {$0?.superview}){
            
            if let responder = view?.next{
                
                if responder.isKind(of: UIViewController.self){
                    
                    return responder as? UIViewController
                }
            }
        }
        return nil
    }
}
