//*******************************************
//**  Copyright  (C)  WCH  2004-2023       **
//**  Web:  http://wch.cn                  **
//*******************************************
//**  iochub libray for iOS/macOS          **
//*******************************************
//
// iocHub接口库 V1.1
// 南京沁恒微电子
// 运行环境: iOS/macOS
//

import UIKit
import DropDown

class ViewController: MH_VC_BaseViewController, UITextViewDelegate, UITextFieldDelegate {
    
    @IBOutlet weak var locationIDLabel: UILabel!
    @IBOutlet weak var receiveTextView: UITextView!
    @IBOutlet weak var sendTextView: UITextView!
    @IBOutlet weak var startButton: UIButton!
    @IBOutlet weak var chooseButton: UIButton!
    @IBOutlet weak var openButton: UIButton!
    @IBOutlet weak var lanDeviceTextField: UITextField!
    @IBOutlet weak var inputDeviceTextField: UITextField!
    @IBOutlet weak var inputSecretTextFiled: UITextField!
    @IBOutlet weak var localSecretTextField: UITextField!
    @IBOutlet weak var topLayoutConstraint: NSLayoutConstraint!
    
    
    @IBOutlet weak var timeTextField: UITextField!
    @IBOutlet weak var showContentButton: UIButton!
    @IBOutlet weak var timeSetButton: UIButton!
    @IBOutlet weak var sendButton: UIButton!
    @IBOutlet weak var fileSendButton: UIButton!
    @IBOutlet weak var secretButton: UIButton!
    
    @IBOutlet weak var receiveBytesLabel: UILabel!
    @IBOutlet weak var receiveSpeedLabel: UILabel!
    
    @IBOutlet weak var sendBytesLabel: UILabel!
    @IBOutlet weak var sendSpeedLabel: UILabel!
    
    private var paramsView:ConfigParamsView?
    let chooseDropDown = DropDown()
    
    var startStatus:Bool = false
    var isFileSend:Bool = false
    
    var deviceList:[String:String] = [:]

    var receiveText = NSMutableString.init()
    let iochub = IOCHubLibrary.shareInstace()

    var sesssionList:[String:Int] = [:]
    var updateTimer: Timer?
    var sendTimer: Timer?
    var sendTimeGap:TimeInterval = 0
    
    var currentReceiveTime:TimeInterval = 0
    var preveReceiveTime:TimeInterval = 0
    var fileSendQueue:OperationQueue?
    var packageSize:UInt32 = 4096
    
    var totalReceiveCount = 0
    var prevRecevieCount = 0
    var totalSendCount = 0
    var prevSendCount = 0
    
    var uploadSpeed: Double = 0.0
    var downloadSpeed: Double = 0.0
    var uploadMetric: String = "KB"
    var downloadMetric: String = "KB"
    
    deinit {
        print("页面已经释放\n");
        self.updateTimer?.invalidate()
        self.sendTimer?.invalidate()
        self.updateTimer = nil
        self.sendTimer = nil
        iochub?.deInit()
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view.
        
        self.receiveTextView.layer.borderColor = UIColor.systemGray5.cgColor
        self.receiveTextView.layer.borderWidth = 1
        
        self.sendTextView.layer.borderColor = UIColor.systemGray5.cgColor
        self.sendTextView.layer.borderWidth = 1
        self.sendTextView.delegate = self
        
        self.timeTextField.delegate = self

        self.addUpdateTimer()
        
        iochub?.delegate = self
        let version = iochub?.getSDKVersion()
        print(version)
    }
    
    @IBAction func configAction(_ sender: Any) {
        if (nil == self.paramsView) {
            self.paramsView = ConfigParamsView.instanceFromNib()
        }
        self.paramsView?.frame = CGRect.init(x: 0, y: 0, width: UIScreen.main.bounds.width, height: UIScreen.main.bounds.height)
        self.view.addSubview(self.paramsView!)
        
        guard let model = iochub?.getConfig() else {return}
        self.paramsView?.addDefaultParams(model)
        self.paramsView?.sureCallBack = {[weak self] params in
            guard let wself = self else {return}
            wself.iochub?.setConfig(params)
            wself.packageSize = params.sendSizePerPackage
        }
    }
    
    @IBAction func startIochubAction(_ sender: Any) {
        if (self.startStatus) {
            _=self.showLoadingWithText("关闭服务....")
            iochub?.stop()
            self.startButton.setTitle("启动", for:.normal)
        }else {
            _=self.showLoadingWithText("启动服务....")
         
            iochub?.start("58.213.74.190:38089")
            self.startButton.setTitle("关闭", for:.normal)
        }
    }
    
    @IBAction func sercretAction(_ sender: Any) {
        self.secretButton.isSelected = !self.secretButton.isSelected
        let model = IOCHubAccessModel()
        model.keySize = Int32(self.localSecretTextField.text?.count ?? 0)
        model.sessionKey = self.localSecretTextField.text ?? ""
        if (self.secretButton.isSelected) {
            model.modeType = 1
        }else {
            model.modeType = 0
        }
        self.iochub?.setAccessMode(model)
    }
    
    
    @IBAction func contentAction(_ sender: Any) {
        self.showContentButton.isSelected = !self.showContentButton.isSelected
    }
    
    @IBAction func timeAction(_ sender: Any) {
        self.timeSetButton.isSelected = !self.timeSetButton.isSelected
    }
    
    @IBAction func showDropListAction(_ sender: Any) {
        if (self.deviceList.count == 0) {
            _=self.showFailedMessage("未发现设备")
            return
        }
        self.chooseDropDown.show()
    }
    
    @IBAction func lanScanAction(_ sender: Any) {
        
//        if (!self.startStatus) {
//            _=self.showFailedMessage("没有启动iochub服务")
//            return
//        }
        
        _=self.showLoadingWithText("扫描中....")
        iochub?.scanLanNodes("", timeout: 1000, callBack: {[weak self] (address, nodeId, size) in
            guard let wself = self else {return}
    
            let ip_port:String = address ?? ""
            let nodeId:String = nodeId ?? ""
            if (nodeId.count > 0 && ip_port.count > 0) {
                wself.deviceList[nodeId] = ip_port
            }
            DispatchQueue.main.async {
                wself.hideHud()
                wself.setupChooseDropDown()
            }
        })
    }
    
    func setupChooseDropDown() {
        chooseDropDown.anchorView = chooseButton
        // By default, the dropdown will have its origin on the top left corner of its anchor view
        // So it will come over the anchor view and hide it completely
        // If you want to have the dropdown underneath your anchor view, you can do this:
        chooseDropDown.bottomOffset = CGPoint(x: 0, y: chooseButton.bounds.height)
        
        // You can also use localizationKeysDataSource instead. Check the docs.
        var dataSource:[String] = [String] ()
        for (key, _) in self.deviceList {
            dataSource.append(key)
        }
        chooseDropDown.dataSource = dataSource
        // Action triggered on selection
        chooseDropDown.selectionAction = { [weak self] (index, item) in
            guard let wself = self else { return }
            wself.lanDeviceTextField.text = item
            wself.openWithDeviceID(item, secret:"")
        }
    }
    
    func openWithDeviceID(_ deviceID:String, secret:String) {
        
        if (deviceID.count == 0) {
            _=self.showFailedMessage("请输入或选择设备")
            return
        }
        
        let sessionFd = self.sesssionList[deviceID] ?? 0
        if (sessionFd != 0) {
            _=self.showFailedMessage("会话已打开请关闭后重新选择")
            return
        }
        
        let ret = iochub?.sessionOpen(deviceID, dstSessionKey: secret) ?? 0
        if (ret == 0) {
            
        }
    }
    

    @IBAction func backGroudAction(_ sender: Any) {
        self.inputDeviceTextField.resignFirstResponder()
        self.inputSecretTextFiled.resignFirstResponder()
        self.timeTextField.resignFirstResponder()
        self.sendTextView.resignFirstResponder()
        self.localSecretTextField.resignFirstResponder()
    }
    
    @IBAction func openAction(_ sender: Any) {
        
        let deviceID = self.inputDeviceTextField.text ?? ""
        let sessionFd = self.sesssionList[deviceID] ?? 0
        if (sessionFd > 0) {
            let ret = iochub?.sessionClose(Int32(sessionFd))
            
            if (ret == 0) {
                _=self.showSuccessMessage("会话关闭成功");
                self.openButton.setTitle("打开会话", for: .normal)
                self.sesssionList.removeValue(forKey: deviceID)
                self.inputDeviceTextField.text = ""
                self.lanDeviceTextField.text = ""
            }else {
                _=self.showSuccessMessage("会话关闭失败");
            }
            
        }else {
            self.openWithDeviceID(self.inputDeviceTextField.text ?? "", secret: self.inputSecretTextFiled.text ?? "")
        }
    }
    
    
//    ASYMMETRIC_NAT      = 0x00,           // 非对称类型
//    FULL_CONE_NAT       = 0x01,           // 全锥型
//    RESTRICTED_NAT      = 0x02,           // IP受限锥型
//    PORT_RESTRICTED_NAT = 0x03,           // 端口受限锥型
//    FIXED_IP_NAT        = 0x04,           // 具有固定IP地址
//    SYMMETRIC_NAT       = 0x10,           // 对称NAT
//    NOT_SUPPORT_UDP     = 0x11,           // 不支持UDP
//    STUN_UNKNOWN        = 0xFF,           // 未知类型
    
    @IBAction func checkNetType(_ sender: Any) {
        let type = iochub?.getNATType()
        switch type{
        case 0x00:
            _=self.showSuccessMessage("非对称类型")
            break
        case 0x01:
            _=self.showSuccessMessage("全锥型")
            break
        case 0x02:
            _=self.showSuccessMessage("IP受限锥型")
            break
        case 0x03:
            _=self.showSuccessMessage("端口受限锥型")
            break
        case 0x04:
            _=self.showSuccessMessage("具有固定IP地址")
            break
        case 0x10:
            _=self.showSuccessMessage("对称NAT")
            break
        case 0x11:
            _=self.showSuccessMessage("不支持UDP")
            break
        case 0xFF:
            _=self.showSuccessMessage("未知类型")
            break
        default:
            _=self.showSuccessMessage("获取网络类型失败")
            break
        }
    }
    
    
    //T_UNKNOWN       = 0x00,                // 未知
    //T_MODE_P2P        = 0x01,                // P2P穿透
    //T_MODE_RELAY    = 0x02,                // 服务转发
    //T_NET_MODE_LAN    = 0x03,                // 局域网
    
    @IBAction func checkCommunicateType(_ sender: Any) {
        
        let deviceID = self.inputDeviceTextField.text ?? ""
        let sessionFd = self.sesssionList[deviceID] ?? 0
        
        let type = iochub?.getTransferType(Int32(sessionFd))
        switch type {
        case 0x00:
            _=self.showSuccessMessage("未知类型")
            break
        case 0x01:
            _=self.showSuccessMessage("P2P穿透")
            break
        case 0x02:
            _=self.showSuccessMessage("服务转发")
            break
        case 0x03:
            _=self.showSuccessMessage("局域网")
            break
        default:
            _=self.showSuccessMessage("获取通讯类型失败")
            break
        }
    }
    
    
    @IBAction func clearReceiveAction(_ sender: Any) {
        self.receiveTextView.text = ""
        self.receiveSpeedLabel.text = "0KB/s"
        self.receiveBytesLabel.text = "0B"
        self.totalReceiveCount = 0
    }
    
    @IBAction func clearSendAction(_ sender: Any) {
        self.sendTextView.text = ""
        self.sendSpeedLabel.text = "0KB/s"
        self.sendBytesLabel.text = "0B"
        self.totalSendCount = 0
    }
    
    @IBAction func sendDataAction(_ sender: Any) {
        self.sendTextView.resignFirstResponder()
        if (self.sendTextView.text.count == 0) {
            _=self.showFailedMessage("请输入发送内容")
            return
        }
        
        let deviceID = self.inputDeviceTextField.text ?? ""
        let sessionFd = self.sesssionList[deviceID] ?? 0
        if (sessionFd == 0) {
            _=self.showFailedMessage("设备未连接")
            return
        }
        
        if (self.timeSetButton.isSelected) {
            
            if (self.sendTimer != nil) {
                self.sendTimer?.invalidate()
                self.sendTimer = nil
            }
            
            self.sendTimeGap = (TimeInterval(self.timeTextField.text!) ?? 0) / 1000
            
            self.sendButton.isSelected = !self.sendButton.isSelected
            if (self.sendButton.isSelected) {
                self.sendButton.setTitle("停止", for: .normal)
                self.addSendTimer()
            }else {
                self.sendButton.setTitle("发送", for: .normal)
            }
            
        }else {
            let text = self.sendTextView.text
            DispatchQueue.global().async {
                let data:Data = text!.data(using: .utf8) ?? Data()
                self.sendContent(sessionFd: Int32(sessionFd), data: data)
            }
        }
    }
    
    @IBAction func sendFileAction(_ sender: Any) {

        if (!self.isFileSend) {
            self.isFileSend = true
            self.fileSendButton.setTitle("停止测试", for: .normal)
            
//            guard let filePath = Bundle.main.url(forResource: "1W_DATA", withExtension: "txt") else {return}
    
            guard let filePath = Bundle.main.path(forResource: "1W_DATA", ofType: "txt") else {return}
            
            let deviceID = self.inputDeviceTextField.text ?? ""
            let sessionFd = self.sesssionList[deviceID] ?? 0
            let offsetBy = Int(self.packageSize)
        
            DispatchQueue.global().async {
                do {
                    
                    guard let fileHandle = try FileHandle.init(forReadingAtPath: filePath) else {return}
                    
                    let fileEndIndex = try fileHandle.seekToEnd()
                    try fileHandle.seek(toOffset: 0)
                    var fileStartIndex:UInt64 = 0
                    while (self.isFileSend) {
                        while fileStartIndex < fileEndIndex {
                            
                            if (!self.isFileSend) {
                                break
                            }
                            
                            autoreleasepool {
                                do {
                                    let sendData = try fileHandle.read(upToCount: offsetBy)
                                    self.sendContent(sessionFd: Int32(sessionFd), data: sendData!)
                                    fileStartIndex = try fileHandle.offset()
                                    if (fileStartIndex == fileEndIndex) {
                                        try fileHandle.seek(toOffset: 0)
                                        fileStartIndex = 0
                                    }
                                    Thread.sleep(forTimeInterval: 0.001)
                                }catch {}
                            }

                        }
                        Thread.sleep(forTimeInterval: 0.001)
                    }
                }catch {}
            }
            
          
        }else {
            self.isFileSend = false
            self.fileSendButton.setTitle("文件测试", for: .normal)
        }
        
        
    }
    
    
    func sendContent(sessionFd:Int32, data:Data) {
        if (data.count == 0) {
            return
        }
        self.writeDataToSocket(sessionFd: sessionFd, data: data)
    }
    
    func writeDataToSocket(sessionFd:Int32, data:Data) {
        let count = data.count
        if (count == 0 || sessionFd == 0) {
            return
        }
        var length:Int = Int(self.iochub?.sessionCouldWriteableLen(sessionFd) ?? 0)
        while length < count {
            length = Int(self.iochub?.sessionCouldWriteableLen(sessionFd) ?? 0)
            Thread.sleep(forTimeInterval: 0.001)
        }
                
        let writeLen = self.iochub?.sessionWrite(sessionFd, write: data) ?? 0
        if (writeLen > 0) {
            self.totalSendCount += Int(writeLen)
        }
      
//        autoreleasepool {
//            var sendData = Data.init(data)
//            let count = data.count
//            let length:Int = Int(self.iochub?.sessionWrite(sessionFd, write: sendData) ?? 0)
//
//            if (length < 0) {
//                if (length != -401) {
//                    print("errorCode:\(length)", length)
//                    return
//                }
//
//                Thread.sleep(forTimeInterval: 0.01)
//                self.writeDataToSocket(sessionFd: sessionFd, data: sendData)
//            }else {
//
//                self.totalSendCount += Int(length)
//                if (length < count) {
//                    let sliceData = Data.init(data)
//                    sendData = sliceData [length ..< count]
//                }else {
//                    sendData = Data.init()
//                }
//                Thread.sleep(forTimeInterval: 0.001)
//                self.writeDataToSocket(sessionFd: sessionFd, data: sendData)
//            }
//        }
    }
    
    func fileDataAddQueue(dataArray:[Data], sessionFd:Int32) {
        if (sessionFd == 0) {
            return
        }
        
        autoreleasepool {

            while (true) {
                for i in 0 ..< dataArray.count {
                    let sendData = dataArray[i]
                    let operation = BlockOperation { [weak self] in
                        guard let wself = self else {return}
                        if (!wself.isFileSend) {
                            wself.fileSendQueue?.cancelAllOperations()
                        }
                        wself.sendContent(sessionFd: sessionFd, data: sendData)
                    }
                   
                    while (self.fileSendQueue!.operations.count > 5) {
                        Thread.sleep(forTimeInterval: 0.001)
                    }
                    
                    self.fileSendQueue?.addOperation(operation)
                }
                
                if (!self.isFileSend) {
                    break
                }
                
                Thread.sleep(forTimeInterval: 0.001)
            }
        }
    }
    
    func textViewDidBeginEditing(_ textView: UITextView) {
        self.topLayoutConstraint.constant = -210
    }
    
    func textViewDidEndEditing(_ textView: UITextView) {
        self.topLayoutConstraint.constant = 0
    }
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        self.topLayoutConstraint.constant = 0
    }
    
    func textFieldDidBeginEditing(_ textField: UITextField) {
        self.topLayoutConstraint.constant = -210
    }
    
    func  addUpdateTimer() {
        
        self.updateTimer = Timer.scheduledTimer(withTimeInterval: 1.0, repeats: true) { _ in
            
            if (self.totalReceiveCount == 0) {
                self.downloadSpeed = 0.0
                self.downloadMetric = "KB"
            }else {
                let currentcount = self.totalReceiveCount
                let deltain: Double = Double(currentcount - self.prevRecevieCount) / 1024.0
                self.prevRecevieCount = currentcount
                
                if (deltain > 1000.0) {
                    self.downloadSpeed = deltain / 1024.0
                    self.downloadMetric = "MB"
                } else {
                    self.downloadSpeed = deltain
                    self.downloadMetric = "KB"
                }
                
                DispatchQueue.main.async {
                    self.receiveBytesLabel.text = "\(self.totalReceiveCount)"
                    self.receiveSpeedLabel.text = String.init(format: "%0.1f%@/s", self.downloadSpeed, self.downloadMetric)
                }
                
            }
            
            if (self.totalSendCount == 0) {
                self.uploadSpeed = 0.0
                self.uploadMetric = "KB"
            }else {
                let currentcount = self.totalSendCount
                let deltaout: Double = Double(currentcount - self.prevSendCount) / 1024.0
                self.prevSendCount = currentcount
                
                if (deltaout > 1000.0) {
                    self.uploadSpeed = deltaout / 1024.0
                    self.uploadMetric = "MB"
                } else {
                    self.uploadSpeed = deltaout
                    self.uploadMetric = "KB"
                }
                
                DispatchQueue.main.async {
                    self.sendBytesLabel.text = "\(self.totalSendCount)"
                    self.sendSpeedLabel.text = String.init(format: "%0.1f%@/s", self.uploadSpeed, self.uploadMetric)
                }
                
            }
        }
        
        RunLoop.current.add(self.updateTimer!, forMode: .common)
    }
    
    func addSendTimer() {
       
        self.sendTimer = Timer.scheduledTimer(withTimeInterval: self.sendTimeGap, repeats: true, block: { _ in
           
            let text = self.sendTextView.text
            let deviceID = self.inputDeviceTextField.text ?? ""
            let sessionFd = self.sesssionList[deviceID] ?? 0
            
            DispatchQueue.global().async {
                let data:Data = text!.data(using: .utf8) ?? Data()
                self.sendContent(sessionFd: Int32(sessionFd), data: data)
            }
        })
        
        RunLoop.current.add(self.sendTimer!, forMode: .common)
    }
    
}

extension ViewController:IOCHubLibraryDelegate {

    func iochubStart(_ status: Int) {
        print("iochubStart status:\(status)");
        self.startStatus = true
        DispatchQueue.main.async {
            self.hideHud()
            self.locationIDLabel.text = self.iochub?.getLocalID()
        }
    }
    
    func iochubStop(_ status: Int) {
        print("iochubStop status:\(status)");
        self.startStatus = false
        DispatchQueue.main.async {
            self.hideHud()
            self.locationIDLabel.text = "--"
        }
    }
    
    func iochubActiveSessionOpen(_ sessionID: Int, deviceID: String?, status: Int) {
        print("iochubActiveSessionOpen sessionID:\(sessionID) deviceID:\(String(describing: deviceID)) status:\(status)");
        DispatchQueue.main.async {
            if (status == 0) {
                if (sessionID > 0) {
                    guard let device = deviceID else {return}
                    self.sesssionList[device] = sessionID
                    self.inputDeviceTextField.text = device
                    
                    _=self.showSuccessMessage("会话打开成功");
                    self.openButton.setTitle("关闭会话", for: .normal)
                }else {
                    _=self.showFailedMessage("打开失败:\(status)")
                    self.openButton.setTitle("打开会话", for: .normal)
                }
            }else {
                _=self.showFailedMessage("打开失败:\(status)")
                self.openButton.setTitle("打开会话", for: .normal)
            }
        }
    }
    
    func iochubUnactiveSessionOpen(_ sessionID: Int, deviceID: String?, status: Int) {
        print("iochubUnactiveSessionOpen sessionID:\(sessionID) deviceID:\(String(describing: deviceID)) status:\(status)");
        
        DispatchQueue.main.async {
            if (status == 0) {
                if (sessionID > 0) {
                    guard let device = deviceID else {return}
                    self.sesssionList[device] = sessionID
                    if (self.inputDeviceTextField.text?.count == 0) {
                        self.inputDeviceTextField.text = device
                    }
                  
                    _=self.showSuccessMessage("有新的会话被打开");
                    self.openButton.setTitle("关闭会话", for: .normal)
                }else {
                    _=self.showFailedMessage("打开失败:\(status)")
                    self.openButton.setTitle("打开会话", for: .normal)
                }
            }else {
                _=self.showFailedMessage("打开失败:\(status)")
                self.openButton.setTitle("打开会话", for: .normal)
            }
        }
    }
    
    func iochubSessionClose(_ sessionID: Int, deviceID: String?, status: Int) {
        print("iochubSessionClose sessionID:\(sessionID) deviceID:\(String(describing: deviceID)) status:\(status)");
        DispatchQueue.main.async {
            if (status == 0) {
                if (sessionID > 0) {
                    guard let device = deviceID else {return}
                    self.sesssionList.removeValue(forKey: device)
                    self.inputDeviceTextField.text = ""
                  
                    _=self.showSuccessMessage("会话已关闭");
                    self.openButton.setTitle("打开会话", for: .normal)
                }
            }else {
                _=self.showFailedMessage("关闭失败:\(status)")
                self.openButton.setTitle("打开会话", for: .normal)
            }
        }
    }
    
    func iochubSessionRead(_ sessionID: Int) {
//        print("iochubSessionRead sessionID:\(sessionID)");
        autoreleasepool {
            guard let data = iochub?.sessionRead(Int32(sessionID), length: 655636) else {return}
            self.totalReceiveCount += data.count
            
            DispatchQueue.main.async {
                if (self.showContentButton.isSelected) {
                    let dataStr = String.init(data: data, encoding: .utf8)
                    self.receiveText.append(dataStr ?? "")
                    self.receiveTextView.text = self.receiveText as String
                }
            }
        }
    }
    
    func iochubSessionTransferTypeChange(_ sessionID: Int, type: Int) {
        let errorStr = "iochubSessionTransferTypeChange sessionID:\(sessionID) type:\(type)"
        DispatchQueue.main.async {
            _=self.showFailedMessage(errorStr)
        }
    }
    
    func iochubSessionException(_ sessionID: Int, status: Int) {
        let errorStr = "iochubSessionException sessionID:\(sessionID) status:\(status)"
        DispatchQueue.main.async {
            _=self.showFailedMessage(errorStr)
        }
    }
    
    func iochubOperationErr(_ status: Int) {
        let errorStr = "iochubOperationErr status:\(status)"
        DispatchQueue.main.async {
            _=self.showFailedMessage(errorStr)
        }
    }
    
    func iochubNetworkChange(_ status: Bool, type: Int) {
        if (status) {
            if (type == 1) {
                _=self.showFailedMessage("有网路,网络类型为WAN或者蜂窝数据")
            }else {
                _=self.showFailedMessage("有网路,网络类型为Wifi")
            }
        }else {
            _=self.showFailedMessage("无网络，请检查网络设置")
        }
    }
}

extension Data {
    func split(by length:Int) -> [Data] {
        var startIndex = self.startIndex
        var results = [Data]()
        while startIndex < self.endIndex {
            let endIndex = self.index(startIndex, offsetBy: length, limitedBy: self.endIndex) ?? self.endIndex
            results.append(self[startIndex..<endIndex])
            startIndex = endIndex
        }
        return results
    }
}

extension String {
    var isHexNumber: Bool {
        filter(\.isHexDigit).count == count
    }
    
    func split(by length: Int) -> [String] {
         var startIndex = self.startIndex
         var results = [Substring]()

         while startIndex < self.endIndex {
             let endIndex = self.index(startIndex, offsetBy: length, limitedBy: self.endIndex) ?? self.endIndex
             results.append(self[startIndex..<endIndex])
             startIndex = endIndex
         }

         return results.map { String($0) }
     }
}
