//*******************************************
//**  Copyright  (C)  WCH  2022-2023       **
//**  Web:  http://wch.cn                  **
//*******************************************
//**  iochub libray for iOS/macOS          **
//*******************************************
//
// iocHub接口库 V1.1
// 南京沁恒微电子
// 运行环境: iOS/macOS
//

#import <Foundation/Foundation.h>
#import "IoCHubModel.h"
/*
 *局域网扫描结果回调Block
 */
typedef void (^LanScanCallBack)(NSString* addrStr, NSString *nodeID, int nodeSize);

/*
 *代理方法
 */
@protocol IOCHubLibraryDelegate <NSObject>
/*
 *启动状态通知
 *status 0 - 成功
 */
- (void)iochubStart:(NSInteger)status;
/*
 *关闭状态通知
 *status 0 - 成功
 */
- (void)iochubStop:(NSInteger)status;
/*
 *主动建立会话通知
 *sessionFd 会话句柄
 *deviceID  设备ID
 *status 0 - 成功
 */
- (void)iochubActiveSessionOpen:(NSInteger)sessionFd deviceID:(NSString *)deviceID status:(NSInteger)status;
/*
 *被动建立会话通知
 *sessionFd 会话句柄
 *deviceID  设备ID
 *status 0 - 成功
 */
- (void)iochubUnactiveSessionOpen:(NSInteger)sessionFd deviceID:(NSString *)deviceID status:(NSInteger)status;
/*
 *会话关闭通知
 *sessionFd 会话句柄
 *deviceID  设备ID
 *status 0 - 成功
 */
- (void)iochubSessionClose:(NSInteger)sessionFd deviceID:(NSString *)deviceID status:(NSInteger)status;
/*
 *缓冲区可读通知
 *sessionFd 会话句柄
 */
- (void)iochubSessionRead:(NSInteger)sessionFd;
/*
 *通讯类型改变通知
 *sessionFd 会话句柄
 *type 1 – P2P 2 – 转发 3 – 局域网
 */
- (void)iochubSessionTransferTypeChange:(NSInteger)sessionFd type:(NSInteger)type;
/*
 *会话异常状态通知
 *sessionFd 会话句柄
 *status - 错误码表
 */
- (void)iochubSessionException:(NSInteger)sessionFd status:(NSInteger)status;
/*
 *错误码通知
 *status - 错误码表
 */
- (void)iochubOperationErr:(NSInteger)status;
/*
 *网络状态改变通知
 *status true - 有网络 false - 无网络
 *type 1 - wan/蜂窝数据 2 - wifi
 */
- (void)iochubNetworkChange:(BOOL)status type:(NSInteger)type;

@end


@interface IOCHubLibrary : NSObject
/*
 *设置代理
 */
@property (nonatomic, weak) id <IOCHubLibraryDelegate> delegate;

/*
 *初始化对象
 */
- (instancetype)init;
/*
 *释放对象
 */
- (void)deInit;
/*
 *单列模式
 */
+ (IOCHubLibrary *)shareInstace;
/*
 *配置IoCHub全局参数，如不进行配置，则使用参数默认值进行配置
 */
- (int)setConfig:(IOCHubCfgModel *)pInitCfg;
/*
 *获取IoCHub当前配置参数值
 */
- (IOCHubCfgModel *)getConfig;
/*
 *设置本机访问模式，用于会话连接
 */
- (int)setAccessMode:(IOCHubAccessModel *)pAccessModeCfg;
/*
 *获取当前会话模式配置。
 */
- (IOCHubAccessModel *)getAccessMode;
/*
 *启动IoCHub
 */
- (int)start:(NSString *)svrAddr;
/*
 *停止IoCHub
 */
- (int)stop;
/*
 *打开会话
 */
- (int)sessionOpen:(NSString *)dstNodeId dstSessionKey:(NSString *)dstSessionKey;
/*
 *关闭会话
 */
- (int)sessionClose:(int)sessionFd;
/*
 *读取会话数据
 */
- (NSData *)sessionRead:(int)sessionFd length:(int)length;
/*
 *缓冲区可写长度
 */
- (int)sessionCouldWriteableLen:(int)sessionFd;
/*
 *写入数据
 */
- (int)sessionWrite:(int)sessionFd writeData:(NSData *)data;
/*
 *获取会话传输类型
 */
- (int)getTransferType:(int)sessionFd;
/*
 *局域网内节点扫描
 */
- (int)scanLanNodes:(NSString *)localNetwork timeout:(int)timeout callBack:(LanScanCallBack)callBack;
/*
 *获取本机节点ID
 */
- (NSString *)getLocalID;
/*
 *获取IoCHub版本号
 */
- (NSString *)getSDKVersion;
/*
 *获取当前本机所处网络环境的NAT类型
 */
- (int)getNATType;
/*
 *获取最后一次的错误码结果
 */
- (int)iochubGetLastError;


@end
